# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
from __future__ import print_function
#
def spherematch(ra1,dec1,ra2,dec2,matchlength,chunksize=None,maxmatch=1,debug=False):
    """Match points on a sphere.

    Parameters
    ----------
    ra1, dec1, ra2, dec2 : ndarray
        The sets of coordinates to match.  Assumed to be in decimal degrees
    matchlength : float
        Two points closer than this separation are matched. Assumed to be in decimal degrees.
    chunksize : float, optional
        Value to pass to chunk assignment.
    maxmatch : int, optional
        Allow up to `maxmatch` matches per coordinate.  Default 1. If set to zero,
        All possible matches will be returned.
    debug : bool, optional

    Returns
    -------
    """
    import numpy as np
    from . import chunks
    from ...goddard.astro import gcirc
    #
    # Set default values
    #
    if chunksize is None:
        chunksize = max(4.0*matchlength,0.1)
    #
    # Initialize chunks
    #
    chunk = chunks(ra1,dec1,chunksize)
    chunk.assign(ra2,dec2,matchlength)
    if debug:
        print("raOffset = {0:7.3f}.".format(chunk.raOffset))
        print(chunk.raBounds)
        print(chunk.decBounds)
        print(chunk.chunkList)
    #
    # Create return arrays
    #
    match1 = list()
    match2 = list()
    distance12 = list()
    for i in range(ra1.size):
        currra = np.fmod(ra1[i]+chunk.raOffset,360.0)
        rachunk,decchunk = chunk.get(currra,dec1[i])
        jmax = len(chunk.chunkList[decchunk][rachunk])
        if jmax > 0:
            for j in range(jmax):
                k = chunk.chunkList[decchunk][rachunk][j]
                sep = gcirc(ra1[i],dec1[i],ra2[k],dec2[k],units=2)/3600.0
                if sep < matchlength:
                    match1.append(i)
                    match2.append(k)
                    distance12.append(sep)
    #
    # Sort distances
    #
    omatch1 = np.array(match1)
    omatch2 = np.array(match2)
    odistance12 = np.array(distance12)
    s = odistance12.argsort()
    #
    # Retain only desired matches
    #
    if maxmatch > 0:
        gotten1 = np.zeros(ra1.size,dtype='i4')
        gotten2 = np.zeros(ra2.size,dtype='i4')
        nmatch = 0
        for i in range(omatch1.size):
            if (gotten1[omatch1[s[i]]] < maxmatch and
                gotten2[omatch2[s[i]]] < maxmatch):
                gotten1[omatch1[s[i]]] += 1
                gotten2[omatch2[s[i]]] += 1
                nmatch += 1
        match1 = np.zeros(nmatch,dtype='i4')
        match2 = np.zeros(nmatch,dtype='i4')
        distance12 = np.zeros(nmatch,dtype='d')
        gotten1[:] = 0
        gotten2[:] = 0
        nmatch = 0
        for i in range(omatch1.size):
            if (gotten1[omatch1[s[i]]] < maxmatch and
                gotten2[omatch2[s[i]]] < maxmatch):
                gotten1[omatch1[s[i]]] += 1
                gotten2[omatch2[s[i]]] += 1
                match1[nmatch] = omatch1[s[i]]
                match2[nmatch] = omatch2[s[i]]
                distance12[nmatch] = odistance12[s[i]]
                nmatch += 1
    else:
        match1 = omatch1[s]
        match2 = omatch2[s]
        distance12 = odistance12[s]
    return (match1,match2,distance12)



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def write_ndarray_to_yanny(filename,datatable,structname='mystruct',enums=dict(),hdr=dict()):
    """Converts a NumPy record array into a new FTCL/yanny file.

    Returns a new yanny object corresponding to the file.

    Parameters
    ----------
    filename : str
        The name of a parameter file.
    datatable : numpy.ndarray
        A NumPy record array containing data that can be copied into a yanny object.
    structname : str, optional
        The name to give the structure in the yanny file.  Defaults to 'MYSTRUCT'.
    enums : dict, optional
        A dictionary containing enum information.  See details above.
    hdr : dict, optional
        A dictionary containing keyword/value pairs for the 'header' of the yanny file.

    Returns
    -------
    par : yanny
        The yanny object resulting from writing the file.

    Examples
    --------
    """
    from . import yanny
    par = yanny(filename,np=True,debug=True)
    par['symbols'] = par.dtype_to_struct(datatable.dtype,structname=structname,enums=enums)
    par[structname.upper()] = datatable
    for key in hdr:
        par[key] = hdr[key]
    par.write(filename)
    return par



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
====
pydl
====

Python replacements for functions that are part of the IDL_ built-in library, or
part of astronomical IDL libraries.  The emphasis is on reproducing results of
the astronomical library functions.  Only the bare minimum of IDL_ built-in
functions are implemented to support this.

.. _IDL: http://www.exelisvis.com/language/en-us/productsservices/idl.aspx
"""
#from __future__ import print_function, division

#this indicates whether or not we are in the package's setup.py
try:
    _ASTROPY_SETUP_
except NameError:
    from sys import version_info
    if version_info[0] >= 3:
        import builtins
    else:
        import __builtin__ as builtins
    builtins._ASTROPY_SETUP_ = False
    del version_info

try:
    from .version import version as __version__
except ImportError:
    __version__ = ''
try:
    from .version import githash as __githash__
except ImportError:
    __githash__ = ''

# set up the test command
def _get_test_runner():
    from astropy.tests.helper import TestRunner
    return TestRunner(__path__[0])

def test(package=None, test_path=None, args=None, plugins=None,
         verbose=False, pastebin=None, remote_data=False, pep8=False,
         pdb=False, coverage=False, open_files=False, **kwargs):
    """
    Run the tests using py.test. A proper set of arguments is constructed and
    passed to `pytest.main`.

    Parameters
    ----------
    package : str, optional
        The name of a specific package to test, e.g. 'io.fits' or 'utils'.
        If nothing is specified all default tests are run.

    test_path : str, optional
        Specify location to test by path. May be a single file or
        directory. Must be specified absolutely or relative to the
        calling directory.

    args : str, optional
        Additional arguments to be passed to `pytest.main` in the `args`
        keyword argument.

    plugins : list, optional
        Plugins to be passed to `pytest.main` in the `plugins` keyword
        argument.

    verbose : bool, optional
        Convenience option to turn on verbose output from py.test. Passing
        True is the same as specifying `-v` in `args`.

    pastebin : {'failed','all',None}, optional
        Convenience option for turning on py.test pastebin output. Set to
        'failed' to upload info for failed tests, or 'all' to upload info
        for all tests.

    remote_data : bool, optional
        Controls whether to run tests marked with @remote_data. These
        tests use online data and are not run by default. Set to True to
        run these tests.

    pep8 : bool, optional
        Turn on PEP8 checking via the pytest-pep8 plugin and disable normal
        tests. Same as specifying `--pep8 -k pep8` in `args`.

    pdb : bool, optional
        Turn on PDB post-mortem analysis for failing tests. Same as
        specifying `--pdb` in `args`.

    coverage : bool, optional
        Generate a test coverage report.  The result will be placed in
        the directory htmlcov.

    open_files : bool, optional
        Fail when any tests leave files open.  Off by default, because
        this adds extra run time to the test suite.  Works only on
        platforms with a working `lsof` command.

    kwargs
        Any additional keywords passed into this function will be passed
        on to the astropy test runner.  This allows use of test-related
        functionality implemented in later versions of astropy without
        explicitly updating the package template.

    See Also
    --------
    pytest.main : py.test function wrapped by `run_tests`.

    """
    test_runner = _get_test_runner()
    return test_runner.run_tests(
        package=package, test_path=test_path, args=args,
        plugins=plugins, verbose=verbose, pastebin=pastebin,
        remote_data=remote_data, pep8=pep8, pdb=pdb,
        coverage=coverage, open_files=open_files, **kwargs)

if not _ASTROPY_SETUP_:

    import os
    from warnings import warn
    from astropy import config

    # add these here so we only need to cleanup the namespace at the end
    config_dir = None

    if not os.environ.get('ASTROPY_SKIP_CONFIG_UPDATE', False):
        config_dir = os.path.dirname(__file__)
        try:
            config.configuration.update_default_config(__package__, config_dir)
        except config.configuration.ConfigurationDefaultMissingError as e:
            wmsg = (e.args[0] + " Cannot install default profile. If you are "
                    "importing from source, this is expected.")
            warn(config.configuration.ConfigurationDefaultMissingWarning(wmsg))
            del e

    del os, warn, config_dir  # clean up namespace

#
# pydl stuff starts here
#
from file_lines import file_lines
from pcomp import pcomp
from smooth import smooth
from uniq import uniq
import goddard
import photoop
import pydlutils
import pydlspec2d

class PydlException(Exception):
    pass

__all__ = ['file_lines', 'pcomp', 'smooth', 'uniq', 'PydlException']



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def sdss_flagname(flagname, flagvalue, concat=False):
    """Return a list of flag names corresponding to the values.

    Parameters
    ----------
    flagname : str
        The name of a bitmask group. Not case-sensitive.
    flagvalue : long
        The value to be converted into bitmask names.
    concat : bool, optional
        If set to ``True``, the list of names is converted to a space-separated string.

    Returns
    -------
    sdss_flagname : str or list
        The names of the bitmasks encoded in `flagvalue`.

    Raises
    ------
    KeyError
        If `flagname` is invalid

    Examples
    --------
    >>> pydl.pydlutils.sdss.sdss_flagname('ANCILLARY_TARGET1',2310346608843161600L)
    ['BRIGHTGAL', 'BLAZGX', 'ELG']
    """
    from . import maskbits
    flagu = flagname.upper()
    bitpos = 0
    retval = list()
    while flagvalue > 0:
        bit = flagvalue % 2
        if bit > 0:
            try:
                f = filter(lambda x: x[1] == bitpos,maskbits[flagu].items())
            except KeyError:
                raise KeyError("Unknown flag group {0}!".format(flagu))
            if len(f) > 0:
                retval.append(f[0][0])
        flagvalue >>= 1
        bitpos += 1
    if concat:
        retval = ' '.join(retval)
    return retval




# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def sdss_flagval(flagname,bitname):
    """Convert bitmask names into values.

    Converts human-readable bitmask names into numerical values.  The inputs
    are not case-sensitive; all inputs are converted to upper case internally.

    Parameters
    ----------
    flagname : str
        The name of a bitmask group.
    bitname : str or list
        The name(s) of the specific bitmask(s) within the `flagname` group.

    Returns
    -------
    sdss_flagval : long
        The value of the bitmask name(s).

    Raises
    ------
    KeyError
        If `flagname` or `bitname` are invalid names.

    Examples
    --------
    >>> pydl.pydlutils.sdss.sdss_flagval('ANCILLARY_TARGET1',['BLAZGX','ELG','BRIGHTGAL'])
    2310346608843161600L
    """
    from . import maskbits
    #
    # Make sure inlabel is a list
    #
    if isinstance(bitname,str):
        bitnames = [bitname.upper()]
    else:
        bitnames = [b.upper() for b in bitname]
    flagu = flagname.upper()
    flagvalue = 0L
    for bit in bitnames:
        if flagu in maskbits:
            if bit in maskbits[flagu]:
                flagvalue += 2L**maskbits[flagu][bit]
            else:
                raise KeyError("Unknown bit label {0} for flag group {1}!".format(bit, flagu))
        else:
            raise KeyError("Unknown flag group {0}!".format(flagu))
    return flagvalue



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def sdss_flagexist(flagname, bitname, flagexist=False):
    """Check for the existence of flags.

    Parameters
    ----------
    flagname : str
        The name of a bitmask group. Not case-sensitive.
    bitname : str or list
        The name(s) of the specific bitmask(s) within the `flagname` group.
    flagexist : bool
        If flagexist is True, return a tuple with the second component indicating
        whether the binary flag named `flagname` exists, even if `bitname` is wrong.

    Returns
    -------
    sdss_flagexist : bool or tuple
        A boolean value or a tuple of bool.
    """
    from . import maskbits
    #
    # Make sure label is a list
    #
    #if isinstance(bitname,str):
    #    bitnames = [bitname.upper()]
    #else:
    #    bitnames = [b.upper() for b in bitname]
    f = False
    l = False
    if flagname.upper() in maskbits:
        f = True
        if bitname in maskbits[flagname.upper()]:
            l = True
    if flagexist:
        return (l,f)
    else:
        return l



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the sdss directory of idlutils.
"""
from default_skyversion import default_skyversion
from sdss_flagexist import sdss_flagexist
from sdss_flagname import sdss_flagname
from sdss_flagval import sdss_flagval
from sdss_objid import sdss_objid
from sdss_sweep_circle import sdss_sweep_circle
from set_maskbits import set_maskbits
#
# Cache the maskbits file.
#
maskbits = set_maskbits()
#
# Remove this from the namespace after use.
#
del set_maskbits
#
# Cache the sweep index
#
sweep_cache = {'star':None,'gal':None,'sky':None}



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the spec1d directory of idlspec2d.
"""
from findspec import findspec
from hmf_gal import hmf_gal
from hmf_solve import hmf_solve
from latest_mjd import latest_mjd
from number_of_fibers import number_of_fibers
from pca_gal import pca_gal
from pca_qso import pca_qso
from pca_solve import pca_solve
from pca_star import pca_star
from plot_eig import plot_eig
from readspec import readspec
from skymask import skymask
from spec_append import spec_append
from spec_path import spec_path
from wavevector import wavevector
#
# Used by findspec
#
findspec_cache = None



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the misc directory in the Goddard utilities.
"""
from cirrange import cirrange
from readcol import readcol



from angles_to_xyz import angles_to_xyz
from current_mjd import current_mjd
from munu_to_radec import munu_to_radec
from radec_to_munu import radec_to_munu
from stripe_to_eta import stripe_to_eta
from stripe_to_incl import stripe_to_incl
from xyz_to_angles import xyz_to_angles



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the math directory of idlutils.
"""
from computechi2 import computechi2
from djs_median import djs_median
from djs_reject import djs_reject
from find_contiguous import find_contiguous



#!/usr/bin/env python
# Licensed under a 3-clause BSD style license - see LICENSE.rst

import sys
import imp
try:
    # This incantation forces distribute to be used (over setuptools) if it is
    # available on the path; otherwise distribute will be downloaded.
    import pkg_resources
    distribute = pkg_resources.get_distribution('distribute')
    if pkg_resources.get_distribution('setuptools') != distribute:
        sys.path.insert(1, distribute.location)
        distribute.activate()
        imp.reload(pkg_resources)
except:  # There are several types of exceptions that can occur here
    from distribute_setup import use_setuptools
    use_setuptools()

import glob
import os
from setuptools import setup, find_packages

#A dirty hack to get around some early import/configurations ambiguities
if sys.version_info[0] >= 3:
    import builtins
else:
    import __builtin__ as builtins
builtins._ASTROPY_SETUP_ = True

import astropy
from astropy.setup_helpers import (register_commands, adjust_compiler,
                                   filter_packages, update_package_files,
                                   get_debug_option)
from astropy.version_helpers import get_git_devstr, generate_version_py

# Use this import to get the docstring
import pydl

# Set affiliated package-specific settings
PACKAGENAME = 'pydl'
DESCRIPTION = 'Astropy affiliated package'
LONG_DESCRIPTION = pydl.__doc__
AUTHOR = 'Benjamin Alan Weaver'
AUTHOR_EMAIL = 'benjamin.weaver@nyu.edu'
LICENSE = 'BSD'
URL = 'http://github.com/weaverba137/pydl'

# VERSION should be PEP386 compatible (http://www.python.org/dev/peps/pep-0386)
VERSION = '0.2.1'

# Indicates if this version is a release version
RELEASE = 'dev' not in VERSION

if not RELEASE:
    VERSION += get_git_devstr(False)

# Populate the dict of setup command overrides; this should be done before
# invoking any other functionality from distutils since it can potentially
# modify distutils' behavior.
cmdclassd = register_commands(PACKAGENAME, VERSION, RELEASE)

# Adjust the compiler in case the default on this platform is to use a
# broken one.
adjust_compiler(PACKAGENAME)

# Freeze build information in version.py
generate_version_py(PACKAGENAME, VERSION, RELEASE, get_debug_option())

# Use the find_packages tool to locate all packages and modules
packagenames = filter_packages(find_packages())

# Treat everything in scripts except README.rst as a script to be installed
scripts = [fname for fname in glob.glob(os.path.join('scripts', '*'))
           if os.path.basename(fname) != 'README.rst']

# Additional C extensions that are not Cython-based should be added here.
extensions = []

# A dictionary to keep track of all package data to install
package_data = {PACKAGENAME: ['data/*']}

# A dictionary to keep track of extra packagedir mappings
package_dirs = {}

# Update extensions, package_data, packagenames and package_dirs from
# any sub-packages that define their own extension modules and package
# data.  See the docstring for setup_helpers.update_package_files for
# more details.
update_package_files(PACKAGENAME, extensions, package_data, packagenames,
                     package_dirs)


setup(name=PACKAGENAME,
      version=VERSION,
      description=DESCRIPTION,
      packages=packagenames,
      package_data=package_data,
      package_dir=package_dirs,
      ext_modules=extensions,
      scripts=scripts,
      requires=['astropy'],
      install_requires=['astropy'],
      provides=[PACKAGENAME],
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      license=LICENSE,
      url=URL,
      long_description=LONG_DESCRIPTION,
      cmdclass=cmdclassd,
      zip_safe=False,
      use_2to3=True,
      classifiers = [ 'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Scientific/Engineering :: Astronomy',
        ],
      )



# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def file_lines(path,compress=False):
    """Replicates the IDL FILE_LINES() function.

    Given a path to a file name or a list of such paths, returns the number of
    lines in the file(s).

    Parameters
    ----------
    path : str or list of str
        Path to a file.  Can be a list of paths.
    compress : bool, optional
        If set to ``True``, assumes that all files in `path` are GZIP compressed.

    Returns
    -------
    file_lines : int or list of int
        The number of lines in `path`.  Returns a list of lengths if a list of
        files is supplied.

    Notes
    -----
    The ``/NOEXPAND_PATH`` option in IDL's FILE_LINES() is not implemented.

    References
    ----------
    http://www.exelisvis.com/docs/FILE_LINES.html

    Examples
    --------
    >>> pydl.file_lines('my-one-line-file.txt')
    1
    """
    scalar = False
    if isinstance(path, str):
        working_path = [path]
        scalar = True
    else:
        working_path = path
    lines = list()
    for filename in working_path:
        if compress:
            import gzip
            with gzip.open(filename) as f:
                lines.append(len(f.readlines()))
        else:
            with open(filename) as f:
                lines.append(len(f.readlines()))
    if scalar:
        return lines[0]
    else:
        return lines
